// cwasm.h : interface of the Redcode Assembler
/////////////////////////////////////////////////////////////////////////////


#ifndef __CWASM__
#define __CWASM__
                

#include "cwdata.h"

                
class CRedcodeAssembler : public CObject
{
	DECLARE_DYNCREATE(CRedcodeAssembler)

public:	
	// Construction & Destruction
	CRedcodeAssembler();
	~CRedcodeAssembler();
	
	// Public Interface
	void			LoadSourceCode(CStringList* pSourceText);
	BOOL			WriteObjectCodeToDisk(const char* pSourcePathName);

	BOOL			Assemble(char* pErrorString = NULL,
							 int*  pWrongLineIndex = NULL);

	CCoreProgram*	GetProgram()	{ return &m_object; }
protected:                                                        
	short	m_nStartLine;		// "start" label definition line 
	short	m_nObjCodeLine;		// generated object code line counter
	
	// Symbolic/Coded Programs
	CStringList*	m_source;
	CCoreProgram	m_object;
	
	// Syntax Controllers
	BOOL CharInSet(const char ch,const char* pSet);
	BOOL IsTokenValidLabel(const char* pToken);
	BOOL IsTokenSignedInteger(const char* pToken);
	CString ExtractNextToken(CString* pLine,
							 const char* pFrontSeps,
							 const char* pRearSeps = NULL);

	int IsRedcodeStatement(const char* pToken);
	BOOL ValidateAddrModes(CCoreCell* pNewCell,
						   char* pErrorString = NULL);
	BOOL AssembleOperand(const char* pToken,
						 const char* pOperand,
						 OP_MODE* pOpMode,
						 OP_DATA* pOpData,
						 char* pErrorString = NULL);
						

	// Assembler Functions
	BOOL AssembleLine(const char* pLine,
					  char* pErrorString = NULL);
	BOOL FirstAssemblingStep(char* pErrorString = NULL,
							 int* pLine = NULL);
	BOOL SecondAssemblingStep(char* pErrorString = NULL,
							  char* pMissingLabel = NULL);
	
	// Cross References Tables
	CMapStringToString	m_EQU_Table;		// EQU labels table
	CMapStringToString	m_RIF_Table;		// l-side labels table
	CMapStringToString	m_CROSS_Table;		// r-side labels table
};



#endif // __CWASM__
