// cwdata.cpp : Core Wars Data Types Implementation
/////////////////////////////////////////////////////////////////////////////


#include "cwdata.h"
#include "stdlib.h"		// _fmalloc function
#include "malloc.h"		// _fmalloc function
                  
                  
/////////////////////////////////////////////////////////////////////////////
// Redcode Symbols

// Instructions
extern const char* COMMANDS[] = {	"DAT",
									"MOV",
									"ADD",
									"SUB",
									"JMP",
									"JMZ",
									"JMN",
									"DJN",
									"CMP",
									"---",
									"SPL",  };

extern const int COMMANDS_ITEMS = sizeof(COMMANDS)/sizeof(COMMANDS[0]);


extern const char ADDR_MODES[] = {	ADDR_IMMEDIATE,
									ADDR_REL_DIRECT,
									ADDR_REL_INDIRECT,
									ADDR_AUTODECREMENTED,
									'\0',	};

extern const int ADDR_MODES_ITEMS = sizeof(ADDR_MODES)/sizeof(ADDR_MODES[0]);

// Field Separators
extern const char FIELD_SEPS[] = {	0x20,		// Space
									0x09,		// Tab Chr
									'\0',	};
							
// Operand Separators
extern const char OPERAND_SEPS[] = {	0x2c,		// Comma
										0x20,		// Space
										0x09,		// Tab Chr
										'\0',	};	

// Comment Flag-Chars
extern const char BEGIN_COMMENT[] = {	0x3b,		// Semicolon
										'\0',	};


/////////////////////////////////////////////////////////////////////////////
// CCoreCell


IMPLEMENT_SERIAL(CCoreCell, CObject, 2)


// Construction
CCoreCell::CCoreCell()
{
	m_cmdModes = (BYTE)0;
	m_opA_data = (OP_DATA)0;
	m_opB_data = (OP_DATA)0;
}

CCoreCell::CCoreCell(CCoreCell& cell)     
{
	// Copy Constructor
	m_cmdModes = cell.m_cmdModes;
	m_opA_data = cell.m_opA_data;
	m_opB_data = cell.m_opB_data;
}

CCoreCell::~CCoreCell()
{
}
	
// Public Operators
BOOL CCoreCell::operator==(CCoreCell& cell)
{
	return ((m_cmdModes==cell.m_cmdModes)
			&& (m_opA_data==cell.m_opA_data)
			&& (m_opB_data==cell.m_opB_data));
}

CCoreCell& CCoreCell::operator=(CCoreCell& cell)     
{
	m_cmdModes = cell.m_cmdModes;
	m_opA_data = cell.m_opA_data;
	m_opB_data = cell.m_opB_data;
	return (*this);
}

// Data Interface	
OP_CODE CCoreCell::GetCommand()
{
	return (OP_CODE)((m_cmdModes & 0xf0)>>4);
}

void CCoreCell::SetCommand(OP_CODE cmd)
{
	m_cmdModes &= 0x0f;		// Reset Previous Command
	m_cmdModes |= ((BYTE)cmd)<<4;
}

OP_MODE CCoreCell::GetOpAMode()
{
	return (OP_MODE)((m_cmdModes & 0x0c)>>2);
}

void CCoreCell::SetOpAMode(OP_MODE mode)
{
	m_cmdModes &= 0xf3;		// Reset Previous Mode
	m_cmdModes |= ((BYTE)mode)<<2;
}

OP_DATA CCoreCell::GetOpAValue()
{
	return m_opA_data;
}

void CCoreCell::SetOpAValue(OP_DATA value)
{
	m_opA_data = value;
}

OP_MODE CCoreCell::GetOpBMode()
{
	return (OP_MODE)(m_cmdModes & 0x03);
}

void CCoreCell::SetOpBMode(OP_MODE mode)
{
	m_cmdModes &= 0xfc;		// Reset Previous Mode
	m_cmdModes |= (BYTE)mode;
}

OP_DATA CCoreCell::GetOpBValue()
{
	return m_opB_data;
}

void CCoreCell::SetOpBValue(OP_DATA value)
{
	m_opB_data = value;
}

// Serialization
void CCoreCell::Serialize(CArchive& ar)
{
	if (ar.IsLoading())
	{
		ar >> m_cmdModes;
		ar >> m_opA_data;
		ar >> m_opB_data;
	}
	else
	{
		ar << m_cmdModes;
		ar << m_opA_data;
		ar << m_opB_data;
	}                     
};


/////////////////////////////////////////////////////////////////////////////
// CCoreProgram

IMPLEMENT_SERIAL(CCoreProgram, CObject, 2)

// Construction
CCoreProgram::CCoreProgram()
{
	m_code.RemoveAll();
	m_startAddress = (CORE_ADDRESS)0;
}

CCoreProgram::~CCoreProgram()
{
	m_code.RemoveAll();
}

CORE_ADDRESS CCoreProgram::GetStartingPoint()
{
	return m_startAddress;
}

void CCoreProgram::SetStartingPoint(CORE_ADDRESS start)
{
	m_startAddress = start;
}
	
void CCoreProgram::AddStatement(CCoreCell* pNewStatement)
{
	m_code.AddTail(pNewStatement);
}

CCoreCell* CCoreProgram::GetStatement(int nIndex)
{
	POSITION pos = m_code.FindIndex(nIndex);
	
	if (pos==NULL)
		return NULL;
	else	
		return (CCoreCell*)(m_code.GetAt(pos));
}

void CCoreProgram::ClearProgram()
{
	while (!m_code.IsEmpty())
		delete m_code.RemoveHead();
}

const CObList* CCoreProgram::GetProgramCode()
{
	return &m_code;
}

int CCoreProgram::GetProgramLength()
{
	return m_code.GetCount();
}

// Serialization
void CCoreProgram::Serialize(CArchive& ar)
{
	if (ar.IsLoading())
	{
		ar >> m_startAddress;
	}
	else
	{
		ar << m_startAddress;
	}                     
	
	m_code.Serialize(ar);
};



