// cwdata.h : Core Wars Data Types
/////////////////////////////////////////////////////////////////////////////


#ifndef __CWDATA__
#define __CWDATA__


#include "afx.h"		// CObject Definition
#include "afxcoll.h"	// CObList Definition

/////////////////////////////////////////////////////////////////////////////
// INI Sections & Entries


#define CORE_SECTION	"Core War Params"
#define SIZE_ENTRY		"Core Size"
#define BATTLES_ENTRY	"Battles"
#define STEPS_ENTRY		"Battle Steps"
#define GAP_ENTRY		"Programs Gap"	
#define TASKS_ENTRY		"Programs Max Tasks"


/////////////////////////////////////////////////////////////////////////////
// Core Wars Constants

// Limits              
#define MAX_CORE_SIZE				(UINT)8192
#define MAX_CORE_WAR_BATTLES		(UINT)1000
#define MAX_CORE_WAR_BATTLE_STEPS	(UINT)50000
#define MAX_CORE_PROGRAMS_GAP		(UINT)2048
#define MAX_CORE_PROGRAMS_LENGTH	(UINT)512
#define MAX_CORE_PROGRAMS_TASKS		(UINT)64

// Scores
#define CORE_WAR_WIN_SCORE		3
#define CORE_WAR_DRAW_SCORE		1
#define CORE_WAR_LOSS_SCORE		0


/////////////////////////////////////////////////////////////////////////////
// Assembler Constants

// Non-Assemblable Instructions
#define EQU_STATEMENT		"EQU"

// Operand Identifiers
#define A_OPERAND_CHR_ID		'A'
#define B_OPERAND_CHR_ID		'B'

// Address Mode Identifiers    
#define ADDR_IMMEDIATE			'#'
#define ADDR_REL_DIRECT			'$'
#define ADDR_REL_INDIRECT		'@'
#define ADDR_AUTODECREMENTED	'<'

// Special Labels
#define START_LABEL				"start"
                                       
// Error Messages                                       
#define ERR_MSG_OPERAND_EXPECTED		"operand expected"                                       
#define ERR_MSG_INVALID_ADDR_MODE		"invalid addressing mode"
#define ERR_MSG_INVALID_LABEL			"invalid label"
#define ERR_MSG_LABEL_EXPECTED			"label expected"
#define ERR_MSG_DUPLICATED_LABEL		"label defined more than once"
#define ERR_MSG_INVALID_START_POS		"invalid starting point position"
#define ERR_MSG_STATEMENT_EXPECTED		"redcode statement expected"
#define ERR_MSG_NUMBER_EXPECTED			"integer expected"
#define ERR_MSG_MEMORY_FULL				"memory full"
#define ERR_MSG_COMMENT_EXPECTED		"comment expected"
#define ERR_MSG_LABEL_NOT_FOUND			"label not found"
#define ERR_MSG_NULL_PROGRAM			"no code produced"
#define ERR_MSG_PROGRAM_TOO_LONG		"program too long"

/////////////////////////////////////////////////////////////////////////////
// Core Wars Data Types

typedef unsigned char	OP_CODE;
typedef unsigned char	OP_MODE;
typedef unsigned short	OP_DATA;
typedef unsigned short	CORE_ADDRESS;


/////////////////////////////////////////////////////////////////////////////
// Single Core Cell
class CCoreCell : public CObject
{
	DECLARE_SERIAL(CCoreCell)

public:
	// Construction
	CCoreCell();
	CCoreCell(CCoreCell& cell);

	~CCoreCell();
	
	// Public Operators
	BOOL operator==(CCoreCell& cell);
	CCoreCell& operator=(CCoreCell& cell);
	
	// Serialization
	void Serialize(CArchive& ar);

	OP_CODE GetCommand();
	void	SetCommand(OP_CODE cmd);

	OP_MODE GetOpAMode();
	void	SetOpAMode(OP_MODE mode);

	OP_DATA GetOpAValue();
	void	SetOpAValue(OP_DATA value);

	OP_MODE GetOpBMode();
	void	SetOpBMode(OP_MODE mode);

	OP_DATA GetOpBValue();
	void	SetOpBValue(OP_DATA value);

protected:
	// Data
	OP_DATA	m_opA_data;
	OP_DATA m_opB_data;
	BYTE	m_cmdModes;	// 0-1 opBmode
						// 2-3 opAmode
						// 4-7 command
};
    

/////////////////////////////////////////////////////////////////////////////
// A Core Program
class CCoreProgram : public CObject
{
	DECLARE_SERIAL(CCoreProgram)

public:
	// Construction
	CCoreProgram();
	~CCoreProgram();

	// Operations
	void			AddStatement(CCoreCell* pNewStatement);
    CCoreCell*		GetStatement(int nIndex);
	
	CORE_ADDRESS	GetStartingPoint();
	void			SetStartingPoint(CORE_ADDRESS start);
	
	void			ClearProgram();
	const CObList*	GetProgramCode();
	int				GetProgramLength();
	
	// Serialization
	virtual void Serialize(CArchive& ar);

protected:
	// Data
	CObList			m_code;
    CORE_ADDRESS	m_startAddress;
};

#endif // __CWDATA__ 

