// cwmars.h : M.A.R.S. interface
/////////////////////////////////////////////////////////////////////////////


#ifndef __CWMARS__
#define __CWMARS__


#include "cwdata.h"
#include "cwtxtout.h"	// Text Output
#include "cwgraphs.h"	// Graphic Output


class CRedcodeSimulator : public CObject
{                                       
DECLARE_DYNCREATE(CRedcodeSimulator)

public:
	// Construction & Destruction
	CRedcodeSimulator();
	~CRedcodeSimulator();
	
	// Public Interface
	BOOL InitializeMARS(int nCoreSize,
						int nWarBattles,
						int nBattleSteps,
						int nProgramsGap,
						int nProgramMaxTasks);

	BOOL LoadFirstProgram(const char* pPathName);
	BOOL LoadSecondProgram(const char* pPathName);

	void SetTextOutputWnd(CDumpWnd* pOutWnd)			{ m_pTextWnd = pOutWnd; }
	void SetGraphicOutputWnd(CGraphCoreWnd* pOutWnd)	{ m_pGraphWnd = pOutWnd; }
	
	// These functions return TRUE if the battle has been suspended
	// in order to let the mainframe to process the  user  messages,
	// FALSE if the dead of one program has ended the core war.
	BOOL StartBattle();	
	BOOL StartCoreWar();
	BOOL ResumeCurrentBattle();	              
	
	void BattleAborted();
	BOOL IsBattleAborted() { return m_isBattleAborted; }
    
	// Response
	UINT  GetStepsDone()	{ return m_stepsDone;	}
	UINT  GetBattlesDone()	{ return m_battlesDone;	}

	int GetPlayerOneScore(int* pWins = NULL,    
						  int* pDraws = NULL, 
						  int* pLosses = NULL);	

	int GetPlayerTwoScore(int* pWins = NULL,
						  int* pDraws = NULL, 
						  int* pLosses = NULL);	

protected:                                  
	// Core War Standards
	int	m_coreSize;
	int	m_warBattles;
	int	m_battleSteps;
	int	m_programsGap;
	int	m_maxTasks;
	
	// Output Windows Pointers
	CDumpWnd*		m_pTextWnd;
	CGraphCoreWnd*	m_pGraphWnd;
	
	// The Core             
	HGLOBAL				m_hCore;
	CCoreCell __far*	m_pCore;

	// Core Data
	UINT		m_battles;
	UINT 		m_stepsDone;
	UINT		m_battlesDone;
	
	UINT		m_currProgramID;	// Program in Execution
	UINT		m_firstPlayerID;	// Program That Started
	UINT		m_stepsPerRound;	// Battle Round Duration	
	BOOL		m_isProgramKilled;
	BOOL		m_isBattleAborted;

	// First Opponent Data
	CString			m_pathP1;		// Object File Name
	CCoreProgram	m_codeP1;		// Program Object Code
	CDWordArray		m_tasksP1;		// Current Active Tasks	
	CORE_ADDRESS	m_loadAddressP1;
	int				m_winsP1;
	int				m_lossesP1;
	int				m_drawsP1;
	int				m_scoreP1;

	// Second Opponent Data
	CString			m_pathP2;
	CCoreProgram	m_codeP2;
	CDWordArray		m_tasksP2;	
	CORE_ADDRESS	m_loadAddressP2;
	int				m_winsP2;
	int				m_lossesP2;
	int				m_drawsP2;
	int				m_scoreP2;

	// Protected Operations
	BOOL ExecuteNextStatement(UINT nProg);

	BOOL LoadCodedProgram(const char* pPathName, UINT nProg);
	BOOL LoadCodeIntoCore(UINT nProg);
	BOOL PrepareBattleField();
	void ResetScores();

	void WriteBattleResults(UINT nWinnerProg);
	void WriteFinalReport();

	CORE_ADDRESS GetAOperandLocation(CORE_ADDRESS currPC);
	CORE_ADDRESS GetBOperandLocation(CORE_ADDRESS currPC);
};


#endif __CWMARS__

