//   **********************************
//  ************************************
//  **                                **
//  **  DUMPWND Implementation File   **
//  **                                ** 
//  ************************************
//   **********************************


#include "resource.h"		// Window Icon
#include "cwtxtout.h" 


/////////////////////////////////////////////////////////////////////////////
// CTextLine

IMPLEMENT_DYNCREATE(CTextLine,CObject)

CTextLine::CTextLine()
{
	m_text.Empty();
	m_Extents = CSize(0,0);
	m_color = 0;
}



/////////////////////////////////////////////////////////////////////////////
// CDumpWnd

IMPLEMENT_DYNCREATE(CDumpWnd,CMDIChildWnd) 
                      
                      
// Default Constructor
CDumpWnd::CDumpWnd() : CMDIChildWnd()
{
	m_HScroll = FALSE;
	m_VScroll = FALSE;
};
  


CDumpWnd::~CDumpWnd()
{
	DeleteContents();
};
  
  
  
/////////////////////////////////////////////////////////////////////////////                       
// Window Creating

BOOL CDumpWnd::Create(const char*   wndTitle,
					  CMDIFrameWnd* pParentFrame,	// = NULL
					  DWORD         wndStyle,		// = 0
					  const CRect&  wndRect,		// = rectDefault
					  COLORREF      textColor)		// = DEFAULT_TEXT_COLOR
{                      
	// Set Correct Defaults
	if (wndStyle == 0)
		wndStyle = WS_CHILD | WS_VISIBLE | WS_OVERLAPPEDWINDOW;
	else
		wndStyle |= WS_CHILD;	// WS_CHILD Style Required!
		
	// Register Class                   
	HICON hIcon = AfxGetApp()->LoadIcon(IDI_TEXTOUTWND);
	HBRUSH hBrush = (HBRUSH)(COLOR_WINDOW + 1);
	HCURSOR hCursor = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
	CString className = AfxRegisterWndClass(CS_NOCLOSE,hCursor,hBrush,hIcon);

	// Create MDIChild Window
	if (!CMDIChildWnd::Create(className,		// Registered Class Name
							  (LPCSTR)wndTitle,	// Window Title
							  wndStyle,			// Window Style
							  wndRect,			// Window Rectangle
                              pParentFrame)) 	// pParentWnd = main frame window	
		return FALSE;											 
		
	// Set Text Metrics	                   
	CClientDC DC(this);
	DC.SetMapMode(MM_TEXT);

	// Solid, BackGround Like Brush
	m_textColor = textColor;
	m_eraseBrush.CreateSolidBrush(DC.GetBkColor());

	// Set Text Attributes
	CFont* pOldFont = (CFont*)DC.SelectStockObject(SYSTEM_FONT); 

	// Get Chars Height
	TEXTMETRIC metrics;
	DC.GetOutputTextMetrics(&metrics);
	
	// Directly Logical Units
	m_textHeight = metrics.tmHeight;

    DC.SelectObject(pOldFont);
	
    // Fill BackGround
	ClearAll();

	return TRUE;
}			 	   



/////////////////////////////////////////////////////////////////////////////                       
// Public Operations

void CDumpWnd::ClearAll()
{
	DeleteContents();
	RemoveHScrollBar();
	RemoveVScrollBar();
	EraseBackGround();
}
              
              
              
void CDumpWnd::InsertLine(const char* newLine, COLORREF color/*=NULL_COLOR*/)
{
	CTextLine *pNewLine = new CTextLine;
	if (pNewLine!=0)
	{
		if (m_firstLine==-1)
			m_firstLine = 0;
        
        pNewLine->m_text = newLine;
		pNewLine->m_color = color;
		m_text.Add(pNewLine);
		m_textLines++;
	
		// Vertical Scroll Bar Needed?
		if ((!m_VScroll) && (m_textLines>m_totalRows))
		{
			// Insert Vertical Scroll Bar
			InsertVScrollBar();
			CheckHScrollBar();
		
			RedrawAllWindow();
		}		                   
		else
			if (m_VScroll)
			{
				// When Adding a New Line, All Document Must
				// Scroll to Let the Last Line Be Visible...
				m_firstLine = m_textLines - m_totalRows;
				CWnd::SetScrollPos(SB_VERT,100*m_firstLine/(m_textLines-m_totalRows));
				RedrawAllWindow();
			}
			else
				// No Scroll Needed
				DrawLine(m_textLines-1);		
	}			
}     
         
                       
                       
/////////////////////////////////////////////////////////////////////////////                       
// Implementation 

void CDumpWnd::DeleteContents()
{
	int i = 0;
	while (i<m_text.GetSize())
	{
		delete m_text[i];
		i++;
	}	
	m_text.RemoveAll();

	m_textLines = 0;  
	m_totalRows = 0;
	m_firstLine = -1;
	m_currentRow = -1;
	
	m_horzPosition = 0;
	m_longestLineSize = CSize(0,0);
}



void CDumpWnd::UpdateWindowExtents()
{
	CClientDC DC(this);
	DC.SetMapMode(MM_TEXT);
 
	// Update Window Extensions Parameters                
	CWnd::GetClientRect(&m_wndArea);
	DC.DPtoLP(&m_wndArea);
	
	if (m_textHeight!=0)
		m_totalRows = (m_wndArea.bottom-m_wndArea.top)/m_textHeight;
}

                       
                   
void CDumpWnd::EraseBackGround()
{
	CClientDC DC(this);
	DC.SetMapMode(MM_TEXT);
	
	if (m_eraseBrush.m_hObject!=NULL)
		DC.FillRect(&m_wndArea,&m_eraseBrush);
}


                       
void CDumpWnd::RedrawAllWindow()
{
	if (m_firstLine!=-1)
	{
    	// Set First Line To Draw
		m_currentRow = -1;
		int line = m_firstLine;     

		while ((m_currentRow<m_totalRows-1) && (line<m_textLines))
		{	        
			DrawLine(line);
			line++;
		}
		CWnd::ValidateRect(NULL);
	}	
}



void CDumpWnd::DrawLine(int lineIndex)
{       
	CTextLine* pLine = (CTextLine*)m_text[lineIndex];        
	m_currentRow++;

	// Get Current Device Context
	CClientDC DC(this);
	DC.SetMapMode(MM_TEXT);
	
	// Set System Font
	CFont* pOldFont = (CFont*)DC.SelectStockObject(SYSTEM_FONT);
                      
	// Set Font Color                      
	COLORREF oldTextColor;
	if (pLine->m_color==NULL_COLOR)                         
	{
		oldTextColor = DC.SetTextColor(m_textColor);
		pLine->m_color = m_textColor;
	}
	else	
		oldTextColor = DC.SetTextColor(pLine->m_color);
                 
	// Horizontal Align
	int x = -m_horzPosition;
	int y = m_currentRow*m_textHeight;     
	
	// Write Current Line 
	CString line = pLine->m_text;
	DC.TextOut(x,y,line);               
    
    // Delete 'Till EOLN
    int xExtent;
    if (pLine->m_Extents.cx==0)
	{
		CSize lineSz = DC.GetOutputTextExtent(line,line.GetLength());
		xExtent = lineSz.cx;
		pLine->m_Extents = lineSz;
		if (lineSz.cx>m_longestLineSize.cx)
		{
			m_longestLineSize = lineSz;
			if (lineSz.cx>(m_wndArea.right-m_wndArea.left))
				InsertHScrollBar();
		}		
	}
	else
		xExtent = pLine->m_Extents.cx;	
		
	CRect delArea(x+xExtent,y,m_wndArea.right+1,y+m_textHeight+1);
	                           
	// Fill With BackGround Color
	DC.FillRect(&delArea,&m_eraseBrush);
    
    // Restore Old Font
	DC.SelectObject(pOldFont);
    DC.SetTextColor(oldTextColor);
}



void CDumpWnd::InsertHScrollBar()
{
	m_HScroll = TRUE;
	CWnd::EnableScrollBar(SB_HORZ);				// Enable Left_Right Arrows
	CWnd::SetScrollRange(SB_HORZ,0,100,FALSE);	// 0-100 Normalized Range
	
	// Initially, Text is Left Aligned
	m_horzPosition = 0;
	m_horzOffset = m_longestLineSize.cx - (m_wndArea.right-m_wndArea.left);
	CWnd::SetScrollPos(SB_HORZ,0);

	// Client Area Has Changed...
	CheckVScrollBar();
}



void CDumpWnd::RemoveHScrollBar()
{
	m_HScroll = FALSE;
	m_horzPosition = 0;
	CWnd::ShowScrollBar(SB_HORZ,FALSE);

	// Client Area Has Changed...
	CheckVScrollBar();
}

    
    
void CDumpWnd::CheckHScrollBar()
{
static CRect prevWndArea;
	
	UpdateWindowExtents();
	
	if (prevWndArea!=m_wndArea)
	{
		// Is Horizontal Scroll Bar Needed?
		if (m_longestLineSize.cx<=(m_wndArea.right-m_wndArea.left))
		{
			if (m_HScroll)
				RemoveHScrollBar();
			return;
		}		

		// Enable Horizontal Scroll Bar
		InsertHScrollBar();
	}	
}



void CDumpWnd::InsertVScrollBar()
{
	m_VScroll = TRUE;
	CWnd::EnableScrollBar(SB_VERT);				// Enable Up-Down Arrows
	CWnd::SetScrollRange(SB_VERT,0,100,FALSE);	// 0-100 Normalized Range

	if ((m_textLines-m_totalRows)!=0)
		CWnd::SetScrollPos(SB_VERT,100*m_firstLine/(m_textLines-m_totalRows));

	// Client Area Has Changed...
	CheckHScrollBar();
}



void CDumpWnd::RemoveVScrollBar()
{
	m_VScroll = FALSE;
	m_firstLine = 0;
	CWnd::ShowScrollBar(SB_VERT,FALSE);
	
	// Client Area Has Changed...
	CheckHScrollBar();
}



void CDumpWnd::CheckVScrollBar()
{
static CRect prevWndArea;
	
	UpdateWindowExtents();
	
	if (prevWndArea!=m_wndArea)
	{
	    prevWndArea = m_wndArea;
		
		// Is Vertical Scroll Bar Needed?
		if (m_totalRows>=m_textLines)
		{
			if (m_VScroll)
				RemoveVScrollBar();
			return;
		}	

		// Try To Fill All the Available Rows 
		if (m_firstLine>m_textLines-m_totalRows)
			m_firstLine = __max(m_textLines-m_totalRows,0);

		InsertVScrollBar();
	}	
}


         
/////////////////////////////////////////////////////////////////////////////                       
// Message Handlers 

BEGIN_MESSAGE_MAP(CDumpWnd, CMDIChildWnd)
	//{{AFX_MSG_MAP(CDumpWnd)
	ON_WM_VSCROLL()
	ON_WM_HSCROLL()
	ON_WM_SIZE()
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



void CDumpWnd::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{   
	switch (nSBCode)
	{
		case SB_BOTTOM			:	m_firstLine = m_textLines-m_totalRows;
									break;

		case SB_LINEDOWN		:	if (m_firstLine<(m_textLines-m_totalRows))
										m_firstLine++;
									break;	

		case SB_PAGEDOWN		:	m_firstLine = __min(m_firstLine+m_totalRows-1,
															m_textLines-m_totalRows);
									break;

		case SB_TOP				:   m_firstLine = 0;
									break;

		case SB_LINEUP			:	if (m_firstLine!=0)
										m_firstLine--;
									break;	

		case SB_PAGEUP			:	m_firstLine = __max(m_firstLine-m_totalRows+1,0);
									break;	
		case SB_THUMBPOSITION	:	
		case SB_THUMBTRACK		:	m_firstLine = (nPos*(m_textLines-m_totalRows)/100);
									break;
	}                               
	
	CWnd::SetScrollPos(SB_VERT,100*m_firstLine/(m_textLines-m_totalRows));
	RedrawAllWindow();
}



void CDumpWnd::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
	switch (nSBCode)
	{
		case SB_LEFT			:	m_horzPosition = 0;
									break;

		case SB_LINELEFT		:	if (m_horzPosition>0)
										m_horzPosition = __max(0,m_horzPosition-HORZ_LINE);
									break;	

		case SB_PAGELEFT		:	if (m_horzPosition>0)
										m_horzPosition = __max(0,m_horzPosition-HORZ_PAGE);
									break;

		case SB_RIGHT			:   m_horzPosition = m_horzOffset;
									break;

		case SB_LINERIGHT		:	if (m_horzPosition<m_horzOffset)
										m_horzPosition = __min(m_horzOffset,m_horzPosition+HORZ_LINE);
									break;	

		case SB_PAGERIGHT		:	if (m_horzPosition<m_horzOffset)								
										m_horzPosition = __min(m_horzOffset,m_horzPosition+HORZ_PAGE);
									break;	
		case SB_THUMBPOSITION	:	
		case SB_THUMBTRACK		:	m_horzPosition = (nPos*m_horzOffset/100);
									break;
	}                               
	
	CWnd::SetScrollPos(SB_HORZ,100*m_horzPosition/m_horzOffset);
	RedrawAllWindow();
}



void CDumpWnd::OnSize(UINT nType, int cx, int cy)
{
	CMDIChildWnd::OnSize(nType, cx, cy);

	CheckVScrollBar();
	CheckHScrollBar();
	EraseBackGround();
	RedrawAllWindow();
}



void CDumpWnd::OnPaint()
{                   
	EraseBackGround();
	RedrawAllWindow();
}
