// cwview.cpp : implementation of the CCwView class
//

#include "stdafx.h"
#include "cw.h"

#include "cwdoc.h"
#include "cwview.h"

#include "cwdlg.h"		// Dialog Windows
#include "cwasm.h"		// Redcode Assembler


#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCwView

IMPLEMENT_DYNCREATE(CCwView, CEditView)

BEGIN_MESSAGE_MAP(CCwView, CEditView)
	//{{AFX_MSG_MAP(CCwView)
	ON_COMMAND(ID_EDIT_CLEAR_ALL, OnEditClearAll)
	ON_COMMAND(ID_VIEW_SETTABSTOPS, OnSetTabStops)
	ON_WM_CREATE()
	ON_COMMAND(ID_EDIT_ASSEMBLE, OnEditAssemble)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCwView construction/destruction

CCwView::CCwView()
{
}

CCwView::~CCwView()
{
}

/////////////////////////////////////////////////////////////////////////////
// CCwView drawing

void CCwView::OnDraw(CDC* pDC)
{
	CCwDoc* pDoc = GetDocument();
}

/////////////////////////////////////////////////////////////////////////////
// CCwView diagnostics

#ifdef _DEBUG
void CCwView::AssertValid() const
{
	CEditView::AssertValid();
}

void CCwView::Dump(CDumpContext& dc) const
{
	CEditView::Dump(dc);
}

CCwDoc* CCwView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CCwDoc)));
	return (CCwDoc*) m_pDocument;
}

#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////
// CCwView Creation

int CCwView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CEditView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// Set Tab Stops To Current Value
	m_tabStops = ((CCwApp*)AfxGetApp())->m_tabStops;
	SetTabStops(m_tabStops*4);
		
	return 0;
}


/////////////////////////////////////////////////////////////////////////////
// Menu Command Handlers

void CCwView::OnEditClearAll()
{
	// To Clear All, First Select All, Then Cut
	GetEditCtrl().SetSel(0,-1);	
	GetEditCtrl().Cut();
}

// Tab Stops Management
void CCwView::OnSetTabStops()
{
	CSetTabStopsDlg dlg;
	dlg.m_tabStops = m_tabStops;
	
	if (dlg.DoModal()==IDOK)
	{
		m_tabStops = dlg.m_tabStops;
		((CCwApp*)AfxGetApp())->m_tabStops = m_tabStops;
		
		// From Chrs To DlgUnits
		SetTabStops(dlg.m_tabStops*4);
	}
}

void CCwView::OnEditAssemble()
{
	CRedcodeAssembler rc_asm;
	
	// Load Source Code
	CStringList text;
	CEdit& rEdit = GetEditCtrl();

	char buffer[256];
	int lineLen, lines = rEdit.GetLineCount();	
	
	for (int i=0; i<lines; i++)
	{
		lineLen = rEdit.GetLine(i,buffer,255);
		
		// Insert String Terminator
		buffer[lineLen] = '\0';
		
		text.AddTail(buffer);
	}

	rc_asm.LoadSourceCode(&text);
	
	// Try To Assemble          
	int wrongLine;
	BOOL done = rc_asm.Assemble(buffer,&wrongLine);
	MessageBeep(-1);
	
	// Save Object Code?
	if (done)
	{
		CAssOkDlg dlg;
		dlg.m_lines = rc_asm.GetProgram()->GetProgramLength();
		dlg.m_start = rc_asm.GetProgram()->GetStartingPoint();

		if (dlg.DoModal()==IDOK)
		{
			CString pathName = GetDocument()->GetPathName();
    	
	    	if (pathName.GetLength()==0)
    		{
	    		AfxMessageBox("Source program must be saved first.",MB_ICONINFORMATION);
    
	    		((CCwDoc*)GetDocument())->OnFileSave();
				pathName = GetDocument()->GetPathName();
    		}
    	
			rc_asm.WriteObjectCodeToDisk(pathName);
		}
	}
	else
	{
		// Some Error Occoured
		CAssErrorDlg dlg;
		dlg.m_line = wrongLine;
		dlg.m_error = buffer;
		
		dlg.DoModal();
				
		// Select Wrong Line
		CEdit& rEdit = GetEditCtrl();
		int firstChr = rEdit.LineIndex(wrongLine - 1);
		int lastChr = rEdit.LineIndex(wrongLine) - 1;
		rEdit.SetSel(firstChr,lastChr);
	}
}
