// mainfrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "cw.h"

#include "mainfrm.h"
#include "cwfrmdlg.h"	// Frame's Dialogs

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_COMMAND(ID_BATTLE_LOAD, OnLoadOpponents)
	ON_COMMAND(ID_BATTLE_SINGLE, OnBattleSingle)
	ON_UPDATE_COMMAND_UI(ID_BATTLE_SINGLE, OnUpdateBattleSingle)
	ON_COMMAND(ID_BATTLE_COREWAR, OnBattleCorewar)
	ON_UPDATE_COMMAND_UI(ID_BATTLE_COREWAR, OnUpdateBattleCorewar)
	ON_COMMAND(ID_BATTLE_STATISTICS, OnBattleStatistics)
	ON_UPDATE_COMMAND_UI(ID_BATTLE_STATISTICS, OnUpdateBattleStatistics)
	ON_COMMAND(ID_BATTLE_PARAMETERS, OnBattleParameters)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// arrays of IDs used to initialize control bars

// toolbar buttons - IDs are command buttons
static UINT BASED_CODE buttons[] =
{
	// same order as in the bitmap 'toolbar.bmp'
	ID_FILE_NEW,
	ID_FILE_OPEN,
	ID_FILE_SAVE,
		ID_SEPARATOR,
	ID_EDIT_CUT,
	ID_EDIT_COPY,
	ID_EDIT_PASTE, 
	ID_EDIT_FIND,
	ID_EDIT_ASSEMBLE,
		ID_SEPARATOR,
	ID_BATTLE_SINGLE,
	ID_BATTLE_COREWAR,	
		ID_SEPARATOR,
	ID_APP_ABOUT,
};

static UINT BASED_CODE indicators[] =
{
	ID_SEPARATOR,			// status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};


/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.Create(this) ||
		!m_wndToolBar.LoadBitmap(IDR_MAINFRAME) ||
		!m_wndToolBar.SetButtons(buttons,
		  sizeof(buttons)/sizeof(UINT)))
	{
		TRACE("Failed to create toolbar\n");
		return -1;		// fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE("Failed to create status bar\n");
		return -1;		// fail to create
	}

	// Set Status Bar 1st Pane Style
	m_wndStatusBar.SetPaneInfo(0,ID_SEPARATOR,SBPS_STRETCH,0);	

	// Set Status Bar Font Size
	LOGFONT fontInfo;
	CFont* pFont = m_wndStatusBar.GetFont();
    pFont->GetObject(sizeof(LOGFONT),&fontInfo);
    
	// Modify Font 
    fontInfo.lfHeight = -8;
    pFont->DeleteObject();
    pFont->CreateFontIndirect(&fontInfo);

	// Load .INI Data
	CWinApp* pApp = AfxGetApp();
	m_coreSize = pApp->GetProfileInt(CORE_SECTION,SIZE_ENTRY,MAX_CORE_SIZE);
	m_warBattles = pApp->GetProfileInt(CORE_SECTION,BATTLES_ENTRY,MAX_CORE_WAR_BATTLES);
	m_battleSteps = pApp->GetProfileInt(CORE_SECTION,STEPS_ENTRY,MAX_CORE_WAR_BATTLE_STEPS);
	m_programsGap = pApp->GetProfileInt(CORE_SECTION,GAP_ENTRY,MAX_CORE_PROGRAMS_GAP);
	m_progMaxTasks = pApp->GetProfileInt(CORE_SECTION,TASKS_ENTRY,MAX_CORE_PROGRAMS_TASKS);

	// Create Output Windows    
	CRect textWndRect(0,0,200,300);
	if (((m_pTextOutputWnd = new CDumpWnd)==0)
		|| (!m_pTextOutputWnd->Create("Output",this,
			WS_CHILD|WS_VISIBLE|WS_OVERLAPPEDWINDOW,textWndRect)))
	{
			AfxMessageBox("Can't create text output window.",MB_ICONSTOP);
			m_pTextOutputWnd = NULL;
	}

	if (((m_pGraphOutputWnd = new CGraphCoreWnd)==0)
		|| (!m_pGraphOutputWnd->Create("Graphics",m_coreSize,
				this,WS_CHILD|WS_VISIBLE|WS_OVERLAPPEDWINDOW)))
	{
			AfxMessageBox("Can't create graphic output window.",MB_ICONSTOP);
			m_pGraphOutputWnd = NULL;
	}

	// Minimize Child Windows
	if (m_pTextOutputWnd!=NULL)
		m_pTextOutputWnd->ShowWindow(SW_SHOWMINIMIZED);

	if (m_pGraphOutputWnd!=NULL)
		m_pGraphOutputWnd->ShowWindow(SW_SHOWMINIMIZED);

	// Initialize MARS Object
	m_validMARS = m_MARS.InitializeMARS(m_coreSize,
										m_warBattles,	
										m_battleSteps,
										m_programsGap,
										m_progMaxTasks);

	// Tell MARS Where Unload Text & Graphic Information
	m_MARS.SetTextOutputWnd(m_pTextOutputWnd);
	m_MARS.SetGraphicOutputWnd(m_pGraphOutputWnd);
	
	// Initialize Flags
	m_firstBattleDone = FALSE;
	m_opponentsLoaded = FALSE;    

	return 0;
}

void CMainFrame::OnDestroy()
{
	// Default Operation
	CMDIFrameWnd::OnDestroy();
}


/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CMDIFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CMDIFrameWnd::Dump(dc);
}

#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers
void CMainFrame::OnLoadOpponents()
{
	CSelectOpponentsDlg dlg;
	
	// Old Opponents
	dlg.m_1stPath = m_1stOpponentPath;
	dlg.m_2ndPath = m_2ndOpponentPath;
	
	if (dlg.DoModal()==IDOK)
	{
		// Try To Open The Files...
		m_opponentsLoaded = m_MARS.LoadFirstProgram(dlg.m_1stPath)
								&& m_MARS.LoadSecondProgram(dlg.m_2ndPath);
		
		if (!m_opponentsLoaded)
		{
			// Some Problem Occoured
			AfxMessageBox("Can't Load Selected Files.",MB_ICONSTOP);
		}
		else
		{
			// Dialog Data Are OK
			m_1stOpponentPath = dlg.m_1stPath;
			m_2ndOpponentPath = dlg.m_2ndPath;
		}
	}
}

void CMainFrame::OnBattleStatistics()
{
	CStatisticsDlg dlg;
	
	// Set Data         
	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char fileName[_MAX_FNAME];
	char ext[_MAX_EXT];
	
	// Extract 1st Player File Name From Path
	_splitpath(m_1stOpponentPath,drive,dir,fileName,ext);
	dlg.m_nameP1 = fileName;

	// Extract 2nd Player File Name From Path
	_splitpath(m_2ndOpponentPath,drive,dir,fileName,ext);
	dlg.m_nameP2 = fileName;

	dlg.m_winsP1 = m_P1wins;
	dlg.m_winsP2 = m_P2wins;
	dlg.m_drawsP1 = m_P1draws;
	dlg.m_drawsP2 = m_P2draws;
	dlg.m_lossesP1 = m_P1losses;
	dlg.m_lossesP2 = m_P2losses;
	dlg.m_scoreP1 = m_P1score;
	dlg.m_scoreP2 = m_P2score;
	
	// Open Window
	dlg.DoModal();
}

void CMainFrame::OnUpdateBattleStatistics(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(m_firstBattleDone);
}

/////////////////////////////////////////////////////////////////////////////
// CoreWar Managers
void CMainFrame::OnBattleSingle()
{
	// Try To Fight...
	BOOL userAbort = FALSE;
		
	if (m_pGraphOutputWnd!=NULL)
		m_pGraphOutputWnd->EraseBackground();

	if (m_MARS.StartBattle())
	{
		// Battle Dialog
		CBattleDlg dlg;

		// Open Dialog
		dlg.Create();
		dlg.ShowWindow(SW_SHOW);
		dlg.SetBattleSteps(m_battleSteps);
		
		do {                                   
			// Update Data
			dlg.SetStepsDone(m_MARS.GetStepsDone());
		
			if (userAbort = dlg.WasAbortButtonPressed())
				m_MARS.BattleAborted();
			
		} while ((!userAbort) && (m_MARS.ResumeCurrentBattle()));
		
		// Close Dialog
		dlg.SetStepsDone(m_MARS.GetStepsDone());
		dlg.DestroyWindow();
	}
		
	if ((userAbort)
		|| m_MARS.IsBattleAborted())
	{
		// Failed
		AfxMessageBox("Battle Aborted.",MB_ICONSTOP);
	}
	else
	{    
		// At Least One Battle Has Been Done...
		m_firstBattleDone = TRUE;

		// Load Scores
		m_P1score = m_MARS.GetPlayerOneScore(&m_P1wins,&m_P1draws,&m_P1losses);
		m_P2score = m_MARS.GetPlayerTwoScore(&m_P2wins,&m_P2draws,&m_P2losses);

		// Open Scores Window
		OnBattleStatistics();
	}
	
	if (m_pGraphOutputWnd!=NULL)
		m_pGraphOutputWnd->EraseBackground();
}

void CMainFrame::OnUpdateBattleSingle(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(m_validMARS && m_opponentsLoaded);
}

void CMainFrame::OnBattleCorewar()
{
	// Try To Fight...
	UINT currBattle;
	BOOL userAbort = FALSE;
		
	if (m_pGraphOutputWnd!=NULL)
		m_pGraphOutputWnd->EraseBackground();

	if (m_MARS.StartCoreWar())
	{
		// Battle Dialog
		CCoreWarDlg dlg;

		// Open Dialog
		dlg.Create();
		dlg.ShowWindow(SW_SHOW); 
		dlg.SetTotalBattles(m_warBattles);
		
		do {                                   
			// Update Data
			if (m_MARS.GetBattlesDone()!=currBattle)
			{	
				currBattle = m_MARS.GetBattlesDone();
				dlg.SetBattlesDone(currBattle + 1);
			}
			
			dlg.SetStepsDone(m_MARS.GetStepsDone());
		
			if (userAbort = dlg.WasAbortButtonPressed())
				m_MARS.BattleAborted();
			
		} while ((!userAbort) && (m_MARS.ResumeCurrentBattle()));
		
		// Close Dialog
		dlg.SetStepsDone(m_MARS.GetStepsDone());
		dlg.DestroyWindow();
	}
		
	if ((userAbort)
		|| m_MARS.IsBattleAborted())
	{
		// Failed
		AfxMessageBox("Battle Aborted.",MB_ICONSTOP);
	}
	else
	{	
		// At Least One Battle Has Been Done...
		m_firstBattleDone = TRUE;

		// Load Scores
		m_P1score = m_MARS.GetPlayerOneScore(&m_P1wins,&m_P1draws,&m_P1losses);
		m_P2score = m_MARS.GetPlayerTwoScore(&m_P2wins,&m_P2draws,&m_P2losses);

		// Open Scores Window
		OnBattleStatistics();
	}
	
	if (m_pGraphOutputWnd!=NULL)
		m_pGraphOutputWnd->EraseBackground();
}

void CMainFrame::OnUpdateBattleCorewar(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(m_validMARS && m_opponentsLoaded);
}


void CMainFrame::OnBattleParameters()
{
	CSettingsDlg dlg;

	// Copy Current Data	
	dlg.m_coreSize = m_coreSize;
	dlg.m_warBattles = m_warBattles;
	dlg.m_battleSteps = m_battleSteps;
	dlg.m_programsGap = m_programsGap;
	dlg.m_progMaxTasks = m_progMaxTasks;
	
	if (dlg.DoModal()==IDOK)
	{
		// Retrieve Validated Data
		m_coreSize = dlg.m_coreSize;
		m_warBattles = dlg.m_warBattles;
		m_battleSteps = dlg.m_battleSteps;
		m_programsGap = dlg.m_programsGap;
		m_progMaxTasks = dlg.m_progMaxTasks;
	
		// Update MARS Object
		m_MARS.InitializeMARS(m_coreSize,
								m_warBattles,
								m_battleSteps,
								m_programsGap,
								m_progMaxTasks);

		// Create a New Graphic Output Window
		if (m_pGraphOutputWnd!=NULL)
			m_pGraphOutputWnd->DestroyWindow();
		
		if (((m_pGraphOutputWnd = new CGraphCoreWnd)==0)
			|| (!m_pGraphOutputWnd->Create("Graphics",m_coreSize,this)))
		{
			// Creation Failed
			AfxMessageBox("Can't create graphic output window.",MB_ICONSTOP);
			m_pGraphOutputWnd = NULL;
		}

		// Tell MARS Where Unload Text & Graphic Information
		m_MARS.SetTextOutputWnd(m_pTextOutputWnd);
		m_MARS.SetGraphicOutputWnd(m_pGraphOutputWnd);

		// Try To Load Previous Programs
		if (!m_1stOpponentPath.IsEmpty())
			m_MARS.LoadFirstProgram(m_1stOpponentPath);

		if (!m_2ndOpponentPath.IsEmpty())
			m_MARS.LoadSecondProgram(m_2ndOpponentPath);
	}
}
	

		
